#!/usr/bin/php
<?php
/*
check_asterisk_pjsip_trunk.php 

Icinga / Nagios check Plugin to check via AMI if a specific PJSIP trunk is connected to Asterisk. 
Classical SIP trunk check won't work with PJSIP any more. 

V.1.0
*/ 

// PHP backward compatibility
if (!function_exists('str_contains')) {
    function str_contains(string $haystack, string $needle): bool
    {
        return '' === $needle || false !== strpos($haystack, $needle);
    }
}

$msg = '';
$err = '';
$return_code = 3;
$return_status = array(0 => 'OK', 1 => 'WARNING', 2 => 'CRITICAL', 3 => 'UNKNOWN');

// We must use double quotes to make Newline working on the commandline!
$usage = "\nParameters missing. Usage:\ncheck_asterisk_pjsip_trunk.php -H <Host> [-P <AMI Port>] -u <AMI Username> ".
  "-p <AMI Password> -E <Trunk Endpoint>\n\n";
// $argstr = var_export($argv, true);

// Parse ommand line
$options = getopt('H:P:u:p:E:S:');

if (empty($options)) exit_unknown($usage);
// print_ka_die($options);

$host = (isset($options['H'])) ? $options['H'] : '';
if ($host == '') exit_unknown("Host parameter -H missing");

$port = (isset($options['P'])) ? $options['P'] : '5038';

$ami_username = (isset($options['u'])) ? $options['u'] : '';
if ($ami_username == '') exit_unknown("AMI Username parameter -u missing");

$ami_password = (isset($options['p'])) ? $options['p'] : '';
if ($ami_password == '') exit_unknown("AMI password parameter -p missing");

$endpoint = (isset($options['E'])) ? $options['E'] : '';
if ($endpoint == '') exit_unknown("Trunk endpoint parameter E missing");

$trunks = GetTrunkStatus();

$found = false;
$status = 'Unknown';
foreach($trunks as $trunk) {
	if ($trunk['endpoint'] == $endpoint) {
		$found = true;
		$status = $trunk['status'];
	}
}

if (!$found) exit_critical("Trunk endpoint ".$endpoint." not found");

if ($status == 'Registered') exit_ok("Trunk endpoint ".$endpoint." is Registered");

exit_critical("Trunk endpoint ".$endpoint." is ".$status);




function exit_ok($str) {
	echo "OK - ".$str."\n";
	exit(0);
}

function exit_warning($str) {
	echo "WARNING - ".$str."\n";
	exit(1);
}

function exit_critical($str) {
	echo "CRITICAL - ".$str."\n";
	exit(2);
}

function exit_unknown($str) {
	echo "UNKNOWN - ".$str."\n";
	exit(3);
}


// Returns array with Trunks and their registration status like so:
// [0] => [uri] => sip:fpbx.de:5060
//				[endpoint] => 18475rds
//				[status] => registered
// [1] => [uri] => sip:sipgate.de:5060
//				[endpoint] => 6bt53922
//				[status] => registered
// 
function GetTrunkStatus () {
	$trunks = array();
	$regArr = false;
	$ActionCmd = 
	"Action: PJSIPShowRegistrationsOutbound\r\n";
	$raw = AMIconnect ($ActionCmd);
	$rawArr = explode('ObjectType: registration', $raw);
	$trash = array_shift($rawArr);			// Remove first entry containing 'Asterisk Call Manager/7.0.3,  Response: Success...
	$i=0;
	foreach ($rawArr as $rawItem) {
		$trunks[$i] = array('host' => GetAmiVal($rawItem, 'ServerUri: '),
									'endpoint' => GetAmiVal($rawItem, 'Endpoint: '),
									'status' => GetAmiVal($rawItem, 'Status: '),
									'type' => 'pjsip');	
		$i++;
	}
	return $trunks;
}

// Returns the value following the name field, e.g. if $raw contains a line like
// 'Response: Success'
// this function searches for the string "Response: ' and returns the rest of the line, e.g. 'Success' if found,
// false otherwise. The blank char in $name is important!
//
function GetAmiVal ($raw='', $name='') {
	// find position of value after name..
	$pos = strpos($raw, $name) + strlen($name);
	// if value is empty. strok ignores the needle at the beginning and returns the next line,
	// so we must correct this unwanted behavior...
	if (substr($raw, $pos, 2) == "\r\n") $ret = '';
	else {
		$ret = strtok(substr($raw, $pos), "\r\n");
	}
	return $ret;
}


// This function makes AMI Action Request to Asterisk and returns raw ASCII result
// or false on error
// $ActionCmd is a string containing action commands like so:
// 	"Action: DBGetTree\r\n".
// 	"Family: cidname\r\n".
// 	"Key: 95\r\n".
// 	"Val: Toilette\r\n";
// 
function AMIconnect ($ActionCmd) {
	global $host, $port, $ami_username, $ami_password;
	$raw = false;
	$cmd = 
	"Action: Login\r\n".
	"Username: ".$ami_username."\r\n".
	"Secret: ".$ami_password."\r\n".
	"\r\n".
	$ActionCmd.
	"\r\n".
	"Action: Logoff\r\n".
	"\r\n";
	$fp = @stream_socket_client('tcp://'.$host.':'.$port, $errno, $errstr, 30);
	if (!$fp) {
		exit("\nAMIconnect Socket Error: ".$errno." - ".$errstr."\n\n");
	}
	else {
		fwrite($fp, $cmd);
		$raw = stream_get_contents($fp);
		fclose($fp);
		if (!str_contains($raw, 'Authentication accepted')) {
			exit("\nAMIconnect AMI Error: ".substr($raw, 0, 80)."\n");
		}
	}
	return $raw;
}






?>
